%{
@Name:ai_ct_encoder.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/18
@Brief:
The Ai and Ct functions of USB-3300, Ai and Counter sync acquisition.
Sync Acquisition Ai0 channel and CT0 100 data point of 10 times.
Only USB-3300 support Counter functions.
You can look at user manual know more about functions or Device info.

USB-3300 only connect USB execution result:
MATLAB is 64-bit.
Dll load success.
Device_number: 1
     0
Device_Index: 0 Device_SN: 66666666 Device_Model: 3323
     0
...
     0
ai_remain_buffer: 1
Ai0 100 data points：
   1.0e-03 *
  arrange 1 to 8
   -0.6251         0   -0.6251   -0.3125   -0.6251   -0.3125   -0.3125   -0.6251
...
  arrange 97 to 100
   -0.6251   -0.6251   -0.6251   -0.3125
ct_remain_buffer: 3
ct0 100 data points：
  arrange 1 to 21
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
...
  arrange 85 to 100
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
     0
     0
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Initialize the array for storing the acquired Ai data.
% Usually the array size need >= acquisition data points size.
% Data type is single (float32 in Python, equivalent in Matlab), it is not modifiable.
% The data points saved to Ai_data_array arrange from opened Ai channel.
Ai_data_array = zeros(1, 2400, 'single');

% Initialize the array for storing the acquired Ct data.
% Usually the array size need >= acquisition data points size.
% Data type is int32.
ct0_data_array = zeros(1, 1000, 'int32');

% Define pointer to Ai_data_array type is singlePtr
Ai_data_array_ptr = libpointer('singlePtr', Ai_data_array);

% Define pointer to CT_data_array type is int32Ptr
ct0_data_array_ptr = libpointer('uint32Ptr', ct0_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% Ai voltage range:
% 10.24 = -10.24 to +10.24V and so on
% Voltage range param support 10.24/5.12/2.56/1.28/0.64
VoltageRange = single(10.24);

% Ai channel sample mode:
% 0=Continuous acquisition or 1=Oneshot, param support 0/1
SamplingMode = uint8(0);

% Ai channel connect type:
% 0=Diff or 1=NRSE, param support 0/1
ConnectionType = uint8(1);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
NumOfDataPoints = uint32(100);

% Configuration values for Ai channels:
% Device model channel number start from 0
ChannelNumber = uint8(0);

% Set channel ON/OFF
ChannelOn = uint8(1);
ChannelOff = uint8(0);

% Ai Sampling period, unit ns, steps 10ns:
% USB-3300 is asynchronous DAQ, single channel max rate is 1M=1000.
% Multichannel each channel share out samplerate, 1M/channel.
% sample rate and period:
% 100k = 10000
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
SamplingPeriod = uint32(1000000);

% Ai acquisition trig source:
% 0=SetUSB3AiSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=with DI, 8=with DO, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
TriggerSource = uint8(0);

% Ai acquisition clock source:
% 0=SetUSB3AiSampleRate(), 1=Conv_In, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/3/4/5/6/7/8
ClockSource = uint8(0);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB3GetAi() func timeout, if timeout return -7.
TimeoutValue = int32(200);

% Counter run mode:
% 0: Counting edge
% 1: Pulse width measurement
% 2: Pulse measurement
% 3: Reserve
% 4: Frequency measurement
% 5: Quadrature encoder measurement
CtRunMode = uint8(5);

% Counter Sample mode:
% 0: Single spot measurement
% 1: Sampling clock buffer continuous acquisition
% 2: Implicit buffer acquisition
CtSampleMode = uint8(1);

% Counter trig source
% 0=SetCtTrigSource(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=with DI, 8=with DO, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
CtTrigSource = uint8(6);

% Counter acquisition clock source:
% 0=SetCtSampleRate(), 1=Conv_In, 2=Ai, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/2/3/4/5/6/7/8
CtConvSource = uint8(2);

% Counter encoder mode:
% 0: Quadrature encoder, X1
% 1: Quadrature encoder, X2
% 2: Quadrature encoder, X4
% 3: Two pulse encoder
CtEncoderMode = uint8(0);

% Counter Z init value
CtZValue = uint32(0);

% For SetCtZPhase() A and B trig Z init(0 is LOW, 1 is HIGH)
CtALev = uint8(0);
CtBLev = uint8(0);

% 0 is disable, 1 is enable
CtZEnable = uint8(0);

% Counter channel
ct_channel_0 = uint8(0);

% Find connected DAQ number
DeviceNumber = calllib('USB3000', 'FindUSB3DAQ');
disp(['Device_number: ', num2str(DeviceNumber)]);

% Open specific device and print return value.
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% Get and display the device serial number and device model.
DeviceSerialNumber = calllib('USB3000', 'USB3ReadDevcieSN', DeviceIndex);
DeviceModel = calllib('USB3000', 'USB3ReadDevcieModel', DeviceIndex);
disp(['Device_Index: ', num2str(DeviceIndex), ' Device_SN: ', num2str(DeviceSerialNumber), ' Device_Model: ', num2str(DeviceModel)]);

% Turn OFF Ai channel 0-8 (default Ai channel 0-8 is ON)
for Channel = 0:7
    ErrorCode = calllib('USB3000', 'SetUSB3AiChanSel', DeviceIndex, Channel, ChannelOff);
    if ErrorCode ~= 0
        disp(ErrorCode);
    end
end

% Turn ON and set input voltage Range for Ai channel 0
for Channel = 0:0
    ErrorCode = calllib('USB3000', 'SetUSB3AiChanSel', DeviceIndex, Channel, ChannelOn);
    disp(ErrorCode);
    ErrorCode = calllib('USB3000', 'SetUSB3AiRange', DeviceIndex, Channel, VoltageRange);
    disp(ErrorCode);
end

% Set Device Ai sample mode
ErrorCode = calllib('USB3000', 'SetUSB3AiSampleMode', DeviceIndex, SamplingMode);
disp(ErrorCode);

% Set Device Ai connect type
ErrorCode = calllib('USB3000', 'SetUSB3AiConnectType', DeviceIndex, ConnectionType);
disp(ErrorCode);

% Set Device sample rate
ErrorCode = calllib('USB3000', 'SetUSB3AiSampleRate', DeviceIndex, SamplingPeriod);
disp(ErrorCode);

% Set Device clock source
ErrorCode = calllib('USB3000', 'SetUSB3AiConvSource', DeviceIndex, ClockSource);
disp(ErrorCode);

% Clear AI FIFO
ErrorCode = calllib('USB3000', 'SetUSB3ClrAiFifo', DeviceIndex);
disp(ErrorCode);

% Set Device trig source
ErrorCode = calllib('USB3000', 'SetUSB3AiTrigSource', DeviceIndex, TriggerSource);
disp(ErrorCode);

% Set counter clock source
ErrorCode = calllib('USB3000', 'SetCtConvSource', DeviceIndex, ct_channel_0, CtConvSource);
disp(ErrorCode);
% Set sample rate, already use Ai sample rate don't need set
% ErrorCode = calllib('USB3000', 'SetCtSampleRate', DeviceIndex, ct_channel_0, CtSampleRate);
% disp(ErrorCode);

% Set counter run mode
ErrorCode = calllib('USB3000', 'SetCtRunMode', DeviceIndex, ct_channel_0, CtRunMode);
disp(ErrorCode);

% Set counter sample mode
ErrorCode = calllib('USB3000', 'SetCtSampleMode', DeviceIndex, ct_channel_0, CtSampleMode);
disp(ErrorCode);

% Set counter trig source together with Ai
ErrorCode = calllib('USB3000', 'SetCtTrigSource', DeviceIndex, ct_channel_0, CtTrigSource);
disp(ErrorCode);

% Set the starting condition of Z index
ErrorCode = calllib('USB3000', 'SetCtZPhase', DeviceIndex, ct_channel_0, CtALev, CtBLev);
disp(ErrorCode);

% Set the value after trig Z function
ErrorCode = calllib('USB3000', 'SetCtZValue', DeviceIndex, ct_channel_0, CtZValue);
disp(ErrorCode);

% Set Z function enable
ErrorCode = calllib('USB3000', 'SetCtZEnable', DeviceIndex, ct_channel_0, CtZEnable);
disp(ErrorCode);

% Clear counter FIFO
ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel_0);
disp(ErrorCode);

% Enable Ai software trig acquisition
ErrorCode = calllib('USB3000', 'SetUSB3AiSoftTrig', DeviceIndex);
disp(ErrorCode);

% Continuous execution 10 times USB3GetAi() and USB3GetCt()
for Iteration = 1:10
    ai_remain_buffer = calllib('USB3000', 'USB3GetAi', DeviceIndex, NumOfDataPoints, Ai_data_array_ptr, TimeoutValue);
    % remain_buffer range 0~2000000
    % USB3GetAi() return the remaining Device FIFO buffer value.
    % FIFO buffer 100 data points save to Ai_data_array_pt.
    % Channel_0 100 data points is Ai_data_array(1:100)
    % If open channel 0~1, Channel_1 100 data points is Ai_data_array(100:200) and so on.
    % If remain_buffer increase you should run USB3GetAi() more quick, let remain_buffer steady.
    disp(['ai_remain_buffer: ', num2str(ai_remain_buffer)]);
    disp(['Ai0 100 data points：']);
    Ai_data_array = Ai_data_array_ptr.value;
    disp(Ai_data_array(1:100));
    
    ct_remain_buffer = calllib('USB3000', 'USB3GetCt', DeviceIndex, ct_channel_0, NumOfDataPoints, ct0_data_array_ptr, TimeoutValue);
    % remain_buffer range 0~2000000
    % USB3GetCt() return the remaining Device FIFO buffer value.
    % FIFO buffer 100 data points save to Ct0_data_array_pt.
    % If remain_buffer increase you should run USB3GetCt() more quick, let remain_buffer steady.
    disp(['ct_remain_buffer: ', num2str(ct_remain_buffer)]);
    disp(['ct0 100 data points：']);
    % Data need change to int32 type for display negative values 
    ct0_data_array = typecast(ct0_data_array_ptr.Value,'int32');
    disp(ct0_data_array(1:100));
    
    % Add your code here to handle data
end

% Disable Ai software trig acquisition, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrTrigger', DeviceIndex);
disp(ErrorCode);

% Clear AI FIFO
ErrorCode = calllib('USB3000', 'SetUSB3ClrAiFifo', DeviceIndex);
disp(ErrorCode);

% Clear CT FIFO
ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel_0);
disp(ErrorCode);

% Close Device
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');