%{
@Name:ao_waveform_sync.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/18
@Brief:
The Ao functions of USB-3000, waveform output.
Ao channel Ao0~Ao3 sync output -1V ~ 1V sine wave.
Device AO Analog voltage range is -10V ~ +10V.
Ao channel is different for modul, Ao_4 or NONE.
You can look at user manual know more about functions or Device info.

USB-3000(Ao_4) only connect USB execution result (Matlab version, similar concept as Python example):
MATLAB is 64-bit.
Dll load success.
     0
     0
Ao0~Ao3 outputting -1V and 1V sine wave, Please Enter to stop.
     0
    "0"    "0"    " AoChannel is 0V"
     0
    "0"    "1"    " AoChannel is 0V"
     0
    "0"    "2"    " AoChannel is 0V"
     0
    "0"    "3"    " AoChannel is 0V"
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init -1 ~ 1V sine wave to Ao_data_array
x = linspace(-2*pi, 2*pi, 1000);
Ao_data_array = single(sin(x));
% Initialize the array for storing the waveform data.
% Data type is single (float32 in Python, equivalent in Matlab), it is not modifiable.

% Define pointer to Ao_data_array type is singlePtr
Ao_data_array_ptr = libpointer('singlePtr', Ao_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% voltage value, range is -10 to +10
voltage_0 = single(0);

% Ao channel sample mode:
% 0=continuous non-loop output to computer buffer
% 1=FIFO output loop decide by SetUSB3AoCycle()
AoSampleMode = uint8(1);

% Ao Sampling period, unit ns, steps 10ns:
% USB-3000 is synchronous Ao, max rate is 100k = 10000.
% sample rate and period:
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
AoSampleRate = uint32(10000000);

% Ao acquisition trig source:
% 0=SetUSB3AoSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 6=with Ai, 7=with Di, 7=with Do, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/6/7/8/9/10/11/12/255
AoTrigSource = uint8(255);

% Ao acquisition clock source:
% 0=SetUSB3AoSampleRate(), 1=Conv_In, 2=Ai, 3=Di, 5~8=AO~3
% param support 0/1/2/3/5/6/7/8
AoConvSource = uint8(0);

% Set waveform output cycle
% 0=infinite loop output
% 1=1 times, 2= 2 times and so on.
% param support 0~4294967295
AoCycle = uint32(0);

% Download to Ao FIFO data number
Num = uint32(length(Ao_data_array));

% Ao_channel_4 sync :0(0000)~0xF(1111) Ao3~Ao0
AoSync = uint8(15);

% Open specific device and print return value.
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% set Ao0~Ao3 channel
for AoChannel = 0:3
    % Disable Ao software trig, clear flag
    ErrorCode = calllib('USB3000', 'SetUSB3ClrAoTrigger', DeviceIndex, AoChannel);
    disp(ErrorCode);
    % Clear FIFO
    ErrorCode = calllib('USB3000', 'SetUSB3ClrAoFifo', DeviceIndex, AoChannel);
    disp(ErrorCode);
    % Set Device Ao sample mode
    ErrorCode = calllib('USB3000', 'SetUSB3AoSampleMode', DeviceIndex, AoChannel, AoSampleMode);
    disp(ErrorCode);
    % Set Device sample rate
    ErrorCode = calllib('USB3000', 'SetUSB3AoSampleRate', DeviceIndex, AoChannel, AoSampleRate);
    disp(ErrorCode);
    % Set Device trig source
    ErrorCode = calllib('USB3000', 'SetUSB3AoTrigSource', DeviceIndex, AoChannel, AoTrigSource);
    disp(ErrorCode);
    % Set Device clock source
    ErrorCode = calllib('USB3000', 'SetUSB3AoConvSource', DeviceIndex, AoChannel, AoConvSource);
    disp(ErrorCode);
    % Set digital output cycle 2 times
    ErrorCode = calllib('USB3000', 'SetUSB3AoCycle', DeviceIndex, AoChannel, AoCycle);
    disp(ErrorCode);
    % Download array data to Ao FIFO
    ErrorCode = calllib('USB3000', 'SetUSB3AoDataFifo', DeviceIndex, AoChannel, Ao_data_array_ptr, Num);
    disp(ErrorCode);
end

% Set Ao sync output wave
ErrorCode = calllib('USB3000', 'SetUSB3AoSync', DeviceIndex, AoSync);
disp(ErrorCode);

% Enable Ao software trig start Ao output wave
ErrorCode = calllib('USB3000', 'SetUSB3GlobalSoftTrig', DeviceIndex);
disp(ErrorCode);

% Pause and wait for user input to stop
input('Ao0~Ao3 outputting -1V and 1V sine wave, Please Enter to stop.');

% Disable Ao software trig, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrGlobalSoftTrig', DeviceIndex);
disp(ErrorCode);

% set Ao0~Ao3 channel
for AoChannel = 0:3
    % Set AOChannel to 0V
    ErrorCode = calllib('USB3000', 'SetUSB3AoImmediately', DeviceIndex, AoChannel, voltage_0);
    disp([ErrorCode, AoChannel, " AoChannel is 0V"]);
    % Clear FIFO
    ErrorCode = calllib('USB3000', 'SetUSB3ClrAoFifo', DeviceIndex, AoChannel);
    disp(ErrorCode);
end

% Close Device and stop Ao wave output
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');