%{
@Name:ct_count_edge.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/19
@Brief:
The Ct functions of USB-3300, Counter acquisition count rise edge.
CT0 100 data point of 10 times, interval of 1s.
Only USB-3300 support Counter functions.single -> Ct0Src.
You can look at user manual know more about functions or Device info.

USB-3300 only connect USB execution result:
MATLAB is 64-bit.
Dll load success.
   0
...
   0
ct_remain_buffer: 1550
ct0 100 data points
arrange 1 to 20
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
...
arrange 81 to 100
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
   0
   0
   0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init Ct_data_array
ct0_data_array = zeros(100, 'int32');
% Usually the array size need >= acquisition data points size.
% Data type is int32, it is not modifiable.
% The data points saved to ct_data_array arrange from get counter channel

% Define pointer to Ct_data_array type is uint32Ptr
ct0_data_array_ptr = libpointer('uint32Ptr', ct0_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% Counter run mode:
% 0: Counting edge
% 1: Pulse width measurement
% 2: Pulse measurement
% 3: Reserve
% 4: Frequency measurement
% 5: Quadrature encoder measurement
CtRunMode = uint8(0);

% Counter Sample mode:
% 0: Single spot measurement
% 1: Sampling clock buffer continuous acquisition
% 2: Implicit buffer acquisition
CtSampleMode = uint8(1);

% Ct Sampling period, unit ns, steps 10ns:
% USB-3300 Counter is asynchronous, max rate is 100k = 10000.
% sample rate and period:
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
CtSampleRate = uint32(10000000);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
Num = uint64(100);

% Counter trig source
% 0=SetCtTrigSource(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=With DI, 8=With DO, 9~12=With AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
CtTrigSource = uint8(0);

% Counter acquisition clock source:
% 0=SetCtSampleRate(), 1=Conv_In, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/3/4/5/6/7/8
CtConvSource = uint8(0);

% Counter channel
ct_channel_0 = uint8(0);

% Count start value
CtValue = uint32(0);

% Set edge level: 1=Rising edge, 2=Falling edge
CtCountEdge = uint8(1);

% Count edge overflow value
CtOverflowValueEdgeCount = uint32(1000);

% start value of trig overflow
CtReloadValueEdgeCount = uint32(0);

% Direction: 0=Decrease, 1=Increase
CtDirEdgeCount = uint8(1);

% Src signal: 0=Internal, 1=Src signal on external terminal
SrcSel = uint8(1);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB3GetAi() func timeout, if timeout return -7.
Timeout = int32(200);

% Open specific device and print return value.
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set counter run mode
ErrorCode = calllib('USB3000', 'SetCtRunMode', DeviceIndex, ct_channel_0, CtRunMode);
disp(ErrorCode);

% Set counter sample mode
ErrorCode = calllib('USB3000', 'SetCtSampleMode', DeviceIndex, ct_channel_0, CtSampleMode);
disp(ErrorCode);

% Set sample rate
ErrorCode = calllib('USB3000', 'SetCtSampleRate', DeviceIndex, ct_channel_0, CtSampleRate);
disp(ErrorCode);

% Set counter trig source
ErrorCode = calllib('USB3000', 'SetCtTrigSource', DeviceIndex, ct_channel_0, CtTrigSource);
disp(ErrorCode);

% Set counter clock source
ErrorCode = calllib('USB3000', 'SetCtConvSource', DeviceIndex, ct_channel_0, CtConvSource);
disp(ErrorCode);

% Set counter single source
ErrorCode = calllib('USB3000', 'SetCtSrcSel', DeviceIndex, ct_channel_0, SrcSel);
disp(ErrorCode);

% Set edge level
ErrorCode = calllib('USB3000', 'SetCtCountEdge', DeviceIndex, ct_channel_0, CtCountEdge);
disp(ErrorCode);

% Set the value of start counter
ErrorCode = calllib('USB3000', 'SetCtValue', DeviceIndex, ct_channel_0, CtValue);
disp(ErrorCode);

% Set count overflow limit value
ErrorCode = calllib('USB3000', 'SetCtOverflowValueEdgeCount', DeviceIndex, ct_channel_0, CtOverflowValueEdgeCount);
disp(ErrorCode);

% Set start value of trig overflow
ErrorCode = calllib('USB3000', 'SetCtReloadValueEdgeCount', DeviceIndex, ct_channel_0, CtReloadValueEdgeCount);
disp(ErrorCode);

% Set counter direction Decrease or Increase
ErrorCode = calllib('USB3000', 'SetCtDirEdgeCount', DeviceIndex, ct_channel_0, CtDirEdgeCount);
disp(ErrorCode);

% clear counter FIFO
ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel_0);
disp(ErrorCode);

% Enable soft Trig, start acquisition counter
ErrorCode = calllib('USB3000', 'SetCtSoftTrig', DeviceIndex, ct_channel_0);
disp(ErrorCode);

% Continuous execution 10 times GetAiChans()
for i = 1:10
    pause(1)
    % USB3GetCt() return the remaining Device FIFO buffer value.
    % FIFO buffer 100 data points save to Ct0_data_array_pt(1:100).
    % If remain_buffer increase you should run USB3GetCt() more quick, let remain_buffer steady.
    ct_remain_buffer = calllib('USB3000', 'USB3GetCt', DeviceIndex, ct_channel_0, Num, ct0_data_array_ptr, Timeout);
    fprintf('ct_remain_buffer: %d\n', ct_remain_buffer);
    fprintf('ct0 100 data points\n');
    % Data need change to int32 type for display negative values 
    ct0_data_array = ct0_data_array_ptr.Value;
    ct0_data_array = typecast(ct0_data_array(1:100),'int32')
    disp(ct0_data_array(1:100));
    % Add your code here can handle data
end

% Disable all trig, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrTrigger', DeviceIndex);
disp(ErrorCode);

% Clear CT FIFO
ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel_0);
disp(ErrorCode);

% Close Device
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');