%{
@Name:ct_encoder.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/19
@Brief:
The Ct functions of USB-3300, Counter acquisition encoder.
Acquisition CT0-CT2 100 encoder data of 30 times.
Only USB - 3300 supports counter functions. singleA -> CtxSrc, singleB -> CtxGate.
You can look at user manual know more about functions or Device info.

USB-3300 only connect USB execution result:
0
...
0
1_times:
0_ct_remain_buffer: 2
0_Ct 100 data points
ct_data_array =
  1100 int32
  arrange 1 to 29
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
...
  arrange 88 to 100
   0   0   0   0   0   0   0   0   0   0   0   0   0
1_ct_remain_buffer: 2
1_Ct 100 data points
...
2_ct_remain_buffer: 2
2_Ct 100 data points
...
  arrange 85 to 100
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
0
0
0
0
0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init Ct_data_array
ct_data_array = zeros(100, 'int32');
% Usually the array size need >= acquisition data points size.
% Data type is int32, it is not modifiable.
% The data points saved to Ct_data_array arrange from get counter channel

% Define pointer to Ct_data_array type is uint32Ptr
ct_data_array_ptr = libpointer('uint32Ptr', ct_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
Num = uint64(100);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB3GetAi() func timeout, if timeout return -7.
Timeout = int32(200);

% counter run mode:
% 0: Counting edge
% 1: Pulse width measurement
% 2: Pulse measurement
% 3: Reserve
% 4: Frequency measurement
% 5: Quadrature encoder measurement
CtRunMode = uint8(5);

% counter Sample mode:
% 0: Single spot measurement
% 1: Sampling clock buffer continuous acquisition
% 2: Implicit buffer acquisition
CtSampleMode = uint8(1);

% Ct Sampling period, unit ns, steps 10ns:
% USB-3300 Counter is asynchronous, max rate is 100k = 10000.
% sample rate and period:
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
CtSampleRate = uint32(1000000);

% counter trig source
% 0=SetCtTrigSource(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=With DI, 8=With DO, 9~12=With AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
CtTrigSource = uint8(255);

% counter acquisition clock source:
% 0=SetCtSampleRate(), 1=Conv_In, 2=Ai, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/2/3/4/5/6/7/8
CtConvSource = uint8(0);

% counter encoder mode:
% 0: Quadrature encoder, X1
% 1: Quadrature encoder, X2
% 2: Quadrature encoder, X4
% 3: Two pulse encoder
CtEncoderMode = uint8(0);

% counter Z init value
CtZValue = uint32(0);

% For SetCtZPhase() A and B trig Z init(0 is LOW, 1 is HIGH)
CtALev = uint8(0);
CtBLev = uint8(0);

% 0 is disable, 1 is enable
CtZEnable = uint8(0);

% Open specific device and print return value.
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set Ct0-Ct2
for ct_channel = 0:2
    % Set counter clock source
    ErrorCode = calllib('USB3000', 'SetCtConvSource', DeviceIndex, ct_channel, CtConvSource);
    disp(ErrorCode);
    % Set sample rate
    ErrorCode = calllib('USB3000', 'SetCtSampleRate', DeviceIndex, ct_channel, CtSampleRate);
    disp(ErrorCode);
    % Set counter run mode
    ErrorCode = calllib('USB3000', 'SetCtRunMode', DeviceIndex, ct_channel, CtRunMode);
    disp(ErrorCode);
    % Set counter sample mode
    ErrorCode = calllib('USB3000', 'SetCtSampleMode', DeviceIndex, ct_channel, CtSampleMode);
    disp(ErrorCode);
    % Set counter trig source together with global
    ErrorCode = calllib('USB3000', 'SetCtTrigSource', DeviceIndex, ct_channel, CtTrigSource);
    disp(ErrorCode);
    % Set the starting condition of Z index
    ErrorCode = calllib('USB3000', 'SetCtZPhase', DeviceIndex, ct_channel, CtALev, CtBLev);
    disp(ErrorCode);
    % Set the value after trig Z function
    ErrorCode = calllib('USB3000', 'SetCtZValue', DeviceIndex, ct_channel, CtZValue);
    disp(ErrorCode);
    % Set Z function enable
    ErrorCode = calllib('USB3000', 'SetCtZEnable', DeviceIndex, ct_channel, CtZEnable);
    disp(ErrorCode);
    % clear counter FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel);
    disp(ErrorCode);
end

% Set global trig, start counter acquisition
ErrorCode = calllib('USB3000', 'SetUSB3GlobalSoftTrig', DeviceIndex);
disp(ErrorCode);

% 30 times Counter acquisition for CT0-CT2
for i = 1:30
    fprintf('%d_times:\n', i);
    for ct_channel = 0:2
        % USB3GetCt() return the remaining Device FIFO buffer value.
        % FIFO buffer 100 data points save to ct_data_array_ptr.
        % If remain_buffer increase you should run USB3GetCt() more quick, let remain_buffer steady.
        ct_remain_buffer = calllib('USB3000', 'USB3GetCt', DeviceIndex, ct_channel, Num, ct_data_array_ptr, Timeout);
        fprintf('%d_ct_remain_buffer: %d\n', ct_channel, ct_remain_buffer);
        fprintf('%d_Ct 100 data points\n', ct_channel);
        % Data need change to int32 type for display negative values 
        ct_data_array = ct_data_array_ptr.Value;
        ct_data_array = typecast(ct_data_array(1:100),'int32')
        disp(ct_data_array);
        % Add your code here can handle data
    end
end

% Disable Ct global trig acquisition, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrGlobalSoftTrig', DeviceIndex);
disp(ErrorCode);

for ct_channel = 0:2
    % Clear CT FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel);
    disp(ErrorCode);
end

% Close Device
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');