%{
@Name:ct_period_frequency.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/20
@Brief:
The Ct functions of USB-3300 for counter acquisition frequency.
CT0 - CT2 use 1 data point to calculate period and frequency, and this process is repeated 100 times.
Only USB-3300 supports the Counter functions with the mode of Single -> CTxGate.
You can refer to the user manual to know more about the functions or device information.

USB-3300 only connect USB execution result:
    0
...
    0
0 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
ct_remain_buffer: 13	ct data: 0
1 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
ct_remain_buffer: 9	ct data: 0
2 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
ct_remain_buffer: 0	ct data: 0
0 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
ct_remain_buffer: 13	ct data: 0
1 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
ct_remain_buffer: 9	ct data: 0
2 _ct_channel:	 period: 0.000000 (s)	 frequency: Inf (Hz)
     0
     0
     0
     0
     0
%}

% Clear the workspace and command window
clear;
clc;

% Check if MATLAB is 32-bit or 64-bit and load the corresponding dynamic link library file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll'; % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll'; % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

% Try to load the dynamic link library
try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Initialize the array used to store counter data, with data type int32 and length 1
ct_data_array = zeros(1, 'int32');
% Usually, the array size should be greater than or equal to the number of acquired data points. The data type of this array is not modifiable, and the data points saved in it are arranged according to the acquisition order of the counter channels.

% Define a pointer to ct_data_array with the type of uint32Ptr
ct_data_array_ptr = libpointer('uint32Ptr', ct_data_array);

% Device index, starting from 0
DeviceIndex = int32(0);

% Counter run mode:
% 0: Counting edge
% 1: Pulse width measurement
% 2: Pulse measurement
% 3: Reserve
% 4: Frequency measurement
% 5: Quadrature encoder measurement
CtRunMode = uint8(4);

% Counter sample mode:
% 0: Single spot measurement
% 1: Sampling clock buffer continuous acquisition
% 2: Implicit buffer acquisition
CtSampleMode = uint8(1);

% Ct sampling period, unit: ns, with steps of 10ns
% USB-3300 Counter is asynchronous, and the max rate is 100k = 10000.
% Examples of sampling rate and period correspondence:
% 10k = 100000
% 1k = 1000000
% 100 = 10000000
% 10 = 100000000
% 1 = 1000000000
CtSampleRate = uint32(1000000000);

% Acquisition amount of data. Each opened Ai channel will get this amount of data. It is recommended that the value is within 10% - 20% of the sampling rate.
Num = uint64(1);

% Counter trig source:
% 0 = SetCtTrigSource(), 1 = Ext_Trig_In, 2 - 5 = Din_0 - 3, 
% 7 = With DI, 8 = With DO, 9 - 12 = With AO - 3, 255 = GlobalSoftTrig
% Parameter supports 0/1/2/3/4/5/7/8/9/10/11/12/255
CtTrigSource = uint8(0);

% Counter acquisition clock source:
% 0 = SetCtSampleRate(), 1 = Conv_In, 3 = Di, 4 = DO, 5 - 8 = AO - 3
% Parameter supports 0/1/3/4/5/6/7/8
CtConvSource = uint8(0);

% Counter channel
ct_channel = uint8(0);

% Internal Src signal period, unit: ns, set in steps of 10ns
CtInternalSrcPeriod = uint32(10);

% Timeout unit: ms. It is recommended that Timeout > (Num * Channel_count / Samplerate) * 1000 to avoid the USB3GetAi() function timeout (if timeout, it returns -7).
Timeout = int32(1000);

% Open the specific device and print the return value
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set parameters for Ct0 - Ct2
for ct_channel = 0:2
    % Set counter run mode
    ErrorCode = calllib('USB3000', 'SetCtRunMode', DeviceIndex, ct_channel, CtRunMode);
    disp(ErrorCode);
    % Set counter sample mode
    ErrorCode = calllib('USB3000', 'SetCtSampleMode', DeviceIndex, ct_channel, CtSampleMode);
    disp(ErrorCode);
    % Set sample rate
    ErrorCode = calllib('USB3000', 'SetCtSampleRate', DeviceIndex, ct_channel, CtSampleRate);
    disp(ErrorCode);
    % Set counter trig source
    ErrorCode = calllib('USB3000', 'SetCtTrigSource', DeviceIndex, ct_channel, CtTrigSource);
    disp(ErrorCode);
    % Set counter clock source
    ErrorCode = calllib('USB3000', 'SetCtConvSource', DeviceIndex, ct_channel, CtConvSource);
    disp(ErrorCode);
    % Set counter internal Period
    ErrorCode = calllib('USB3000', 'SetCtInternalSrcPeriod', DeviceIndex, ct_channel, CtInternalSrcPeriod);
    disp(ErrorCode);
    % Clear counter FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel);
    disp(ErrorCode);
    % Enable soft Trig, start acquisition counter
    ErrorCode = calllib('USB3000', 'SetCtSoftTrig', DeviceIndex, ct_channel);
    disp(ErrorCode);
end

% Continuously execute USB3GetCt() for Ct0 - CT2, 100 times
for i = 1:100
    for ct_channel = 0:2
        % USB3GetCt() returns the remaining Device FIFO buffer value.
        % FIFO buffer 1 data point is saved to Ct_data_array_ptr.
        % If the remain_buffer increases, you should run USB3GetCt() more quickly to keep it steady.
        ct_remain_buffer = calllib('USB3000', 'USB3GetCt', DeviceIndex, ct_channel, Num, ct_data_array_ptr, Timeout);
        fprintf('ct_remain_buffer: %d\tct data: %d\n', ct_remain_buffer, ct_data_array(1));

        % Calculate period and frequency and then print them
        ct_data_array = ct_data_array_ptr.Value;
        ct_data_array(1) = typecast(ct_data_array(1),'int32');
        period = double(ct_data_array(1)) / 100000000;
        frequency = 1 / period;
        fprintf('%d _ct_channel:\t period: %f (s)\t frequency: %f (Hz)\n', ct_channel, period, frequency);
        % Add your code here to handle the data
    end
end

% Disable all trig, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrTrigger', DeviceIndex);
disp(ErrorCode);

for channel = 0:2
    % Clear CT FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, channel);
    disp(ErrorCode);
end

% Close the device
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');