%{
@Name:ct_single_type_pulse_width.m
@Author: Smacq
@Copyright: Smacq
@Date: 2024/12/20
@Brief:
The Ct functions of USB-3300, Counter acquisition pulse width.
CT0~CT2 1 data point calculate Single type pulse width of 100 times.
Only USB-3300 support Counter functions, Single -> CTxGate.
You can look at user manual know more about functions or Device info.

USB-3300 only connect USB execution result:
    0
...
    0
0 _ct_channel:	 ct_remain_buffer: 0	ct data: 0
pulse_width: 0.000000 (s)
1 _ct_channel:	 ct_remain_buffer: 26	ct data: 0
pulse_width: 0.000000 (s)
2 _ct_channel:	 ct_remain_buffer: 5	ct data: 0
pulse_width: 0.000000 (s)
...
0 _ct_channel:	 ct_remain_buffer: 0	ct data: 0
pulse_width: 0.000000 (s)
1 _ct_channel:	 ct_remain_buffer: 26	ct data: 0
pulse_width: 0.000000 (s)
2 _ct_channel:	 ct_remain_buffer: 5	ct data: 0
pulse_width: 0.000000 (s)
     0
     0
     0
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB3000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB3000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB3000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB3000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init Ct_data_array
ct_data_array = zeros(1, 'int32');
% Usually the array size need >= acquisition data points size.
% Data type is int32, it is not modifiable.
% The data points saved to Ct_data_array arrange from get counter channel

% Define pointer to Ct_data_array type is uint32Ptr
ct_data_array_ptr = libpointer('uint32Ptr', ct_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% Counter run mode:
% 0: Counting edge
% 1: Pulse width measurement
% 2: Pulse measurement
% 3: Reserve
% 4: Frequency measurement
% 5: Quadrature encoder measurement
CtRunMode = uint8(1);

% Counter Sample mode:
% 0: Single spot measurement
% 1: Sampling clock buffer continuous acquisition
% 2: Implicit buffer acquisition
CtSampleMode = uint8(1);

% Ct Sampling period, unit ns, steps 10ns:
% USB-3300 Counter is asynchronous, max rate is 100k = 10000.
% sample rate and period:
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
CtSampleRate = uint32(1000000000);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
Num = uint64(1);

% Counter trig source
% 0=SetCtTrigSource(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=With DI, 8=With DO, 9~12=With AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
CtTrigSource = uint8(0);

% Counter acquisition clock source:
% 0=SetCtSampleRate(), 1=Conv_In, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/3/4/5/6/7/8
CtConvSource = uint8(0);

% Counter channel
ct_channel = uint8(0);

% measure pulse type: 1=Positive/0=negative
StartEdge = uint8(1);

% Internal Src signal period, in ns, set in steps of 10ns
CtInternalSrcPeriod = uint32(10);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB3GetAi() func timeout, if timeout return -7.
Timeout = int32(1000);

% Open specific device and print return value.
ErrorCode = calllib('USB3000', 'USB3OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set Ct0~Ct2
for ct_channel = 0:2
    % Set counter run mode
    ErrorCode = calllib('USB3000', 'SetCtRunMode', DeviceIndex, ct_channel, CtRunMode);
    disp(ErrorCode);
    % Set counter sample mode
    ErrorCode = calllib('USB3000', 'SetCtSampleMode', DeviceIndex, ct_channel, CtSampleMode);
    disp(ErrorCode);
    % Set sample rate
    ErrorCode = calllib('USB3000', 'SetCtSampleRate', DeviceIndex, ct_channel, CtSampleRate);
    disp(ErrorCode);
    % Set counter trig source
    ErrorCode = calllib('USB3000', 'SetCtTrigSource', DeviceIndex, ct_channel, CtTrigSource);
    disp(ErrorCode);
    % Set counter clock source
    ErrorCode = calllib('USB3000', 'SetCtConvSource', DeviceIndex, ct_channel, CtConvSource);
    disp(ErrorCode);
    % Set counter internal Period
    ErrorCode = calllib('USB3000', 'SetCtInternalSrcPeriod', DeviceIndex, ct_channel, CtInternalSrcPeriod);
    disp(ErrorCode);
    % Set SetCtPulseWidthStartEdge
    ErrorCode = calllib('USB3000', 'SetCtPulseWidthStartEdge', DeviceIndex, ct_channel, StartEdge);
    disp(ErrorCode);
    % clear counter FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel);
    disp(ErrorCode);
    % Enable soft Trig, start acquisition counter
    ErrorCode = calllib('USB3000', 'SetCtSoftTrig', DeviceIndex, ct_channel);
    disp(ErrorCode);
end

% Continuous execution USB3GetCt() Ct0~CT2 of 100 times
for i = 1:100
    for ct_channel = 0:2
        % USB3GetCt() return the remaining Device FIFO buffer value.
        % FIFO buffer 1 data points save to Ct_data_array_ptr.
        % If remain_buffer increase you should run USB3GetCt() more quick, let remain_buffer steady.
        ct_remain_buffer = calllib('USB3000', 'USB3GetCt', DeviceIndex, ct_channel, Num, ct_data_array_ptr, Timeout);
        fprintf('%d _ct_channel:\t ct_remain_buffer: %d\tct data: %d\n', ct_channel, ct_remain_buffer, ct_data_array(1));
        % Calculate pulse width
        ct_data_array = ct_data_array_ptr.Value;
        ct_data_array(1) = typecast(ct_data_array(1),'int32');
        pulse_width = double(ct_data_array(1)) / 100000000;
        fprintf('pulse_width: %f (s)\n', pulse_width);
        % Add your code here can handle data
    end
end

% Disable all trig, clear flag
ErrorCode = calllib('USB3000', 'SetUSB3ClrTrigger', DeviceIndex);
disp(ErrorCode);

for ct_channel = 0:2
    % Clear CT FIFO
    ErrorCode = calllib('USB3000', 'SetCtClrFifo', DeviceIndex, ct_channel);
    disp(ErrorCode);
end

% Close Device
ErrorCode = calllib('USB3000', 'USB3CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB3000');